#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Foot Print
#      By:	sdmitch
#   Usage:	Create an outline of buildings and other surface features
#	 Note:	Needs some way to remove duplicate edges caused by top and bottom edges of features.
#			
#    Date:	Jul 2011
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
    UI.menu('Plugins').add_item('Foot Print') { SDM_FP.footprint }
end
#
file_loaded(filename)
#
module SDM_FP

	def SDM_FP.footprint
		@Mod=Sketchup.active_model; @Ent=@Mod.entities; @Sel=@Mod.selection
		@Mod.start_operation "Foot Print"
		@edges=[]; @bottom=1e6; 
		SDM_FP.get_edges(@Sel); 
		@bottom=@bottom.floor - 1; @Sel.clear
		plane = [Geom::Point3d.new(0, 0 ,@bottom), Geom::Vector3d.new(0,0,1)]
		for edg in @edges
			p1=edg.start.position.project_to_plane plane
			p2=edg.end.position.project_to_plane plane
			edge=@Ent.add_line(p1,p2)
			@Sel.add edge if edge
		end
		save=@Sel.to_a; @Sel.clear
		save.each{|e| e.find_faces}
		save.each{|e| @Sel.add(e) if e.faces.length>1}
		@Ent.erase_entities  @Sel
		# # add faces to connect structure to outline
		for edg in @edges
			p1=edg.start.position.project_to_plane plane
			p2=edg.end.position.project_to_plane plane
			face = @Ent.add_face(edg.start.position,p1,p2,edg.end.position)
			face.back_material=face.material="red"
		end
		# # ===============================
		@Mod.commit_operation
	end
		
	def SDM_FP.get_edges(ents)
		ents.each do |ent|
			if ent.is_a? Sketchup::Group
				tr=Geom::Transformation.new(ent.bounds.min)
				ent.entities.each do |goc|
					if goc.is_a? Sketchup::Face
						if goc.normal.z  < 0.0 then
							edges=goc.edges
							edges.each do |edg|
								p1=edg.start.position.transform tr
								p2=edg.end.position.transform tr
								edge =  @Ent.add_line(p1,p2)
								 if edge
									@edges << edge if !@edges.include? edge
									minz = [edge.start.position.z,edge.end.position.z].min
									@bottom = minz if @bottom > minz
								end
							end
						end
					end
				end
			elsif ent.is_a? Sketchup::ComponentInstance
				tr=Geom::Transformation.new(ent.bounds.min)
				ent.definition.entities.each do |goc|
					if goc.is_a? Sketchup::Face
						if goc.normal.z  < 0.0 then
							edges=goc.edges
							edges.each do |edg|
								p1=edg.start.position.transform tr
								p2=edg.end.position.transform tr
								edge =  @Ent.add_line(p1,p2)
								 if edge
									@edges << edge if !@edges.include? edge
									minz = [edge.start.position.z,edge.end.position.z].min
									@bottom = minz if @bottom > minz
								end
							end
						end
					end
				end
			elsif ent.is_a? Sketchup::Face
				if ent.normal.z < 0.0 then
					edges = ent.edges
					edges.each do |edg|
						p1=edg.start.position
						p2=edg.end.position
						edge =  @Ent.add_line(p1,p2)
						 if edge
							@edges << edge if !@edges.include? edge
							minz = [edge.start.position.z,edge.end.position.z].min
							@bottom = minz if @bottom > minz
						end
					end
				end
			end
		end
	end
	
end

